SystemOrganization addCategory: #QuickNotes!

Model subclass: #QuickNotes
	instanceVariableNames: 'searchText lastSearchText keyStringsListed selectedKeyStringIndex currentNote initialContent notesDB mainWindow contentEditor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuickNotes'!

QuickNotes class
	instanceVariableNames: 'uniqueInstance'!

!QuickNotes commentStamp: '<historical>' prior: 0!
A QuickNotes is Notational Velocity clone.

Instance Variables
	searchText 
	lastSearchText 
	keyStringsListed 
	selectedKeyStringIndex 
	currentNote 
	initialContent 
	notesDB 
	mainWindow 
	contentEditor

Todos:
-----------
+ escape single quotes
+ auto save note DONE
+ change keyboard focus after new note creation
+ Rename note
+ Auto IDs in DB
+ Don't allow to add a Note with an empty key string
+ Use PluggableTextMorph>>acceptOnFocusChange
+ Make QuickNotes a Singleton
- Include date in note
- Show preview of note in list
- Sort listed notes
- Umlaute don't work
- select text of new content automatically
- Correct timestamp (+02:00)
- Resize list and content windows or make resizeable!

QuickNotes class
	instanceVariableNames: 'uniqueInstance'!

!QuickNotes class methodsFor: 'initialize-release' stamp: 'ArkadiuszBaranowski 8/14/2013 14:22'!
delete
	"comment stating purpose of message"

	uniqueInstance := nil! !

!QuickNotes class methodsFor: 'initialize-release' stamp: 'ArkadiuszBaranowski 8/14/2013 14:28'!
new

	uniqueInstance ifNil: [
		uniqueInstance := super new.
	 ].

	! !

!QuickNotes class methodsFor: 'initialize-release' stamp: 'ArkadiuszBaranowski 8/14/2013 14:28'!
open

	self new.
	uniqueInstance mainWindow openInWorld.
! !

!QuickNotes class methodsFor: 'initialize-release' stamp: 'ArkadiuszBaranowski 6/13/2014 13:06'!
shutDown
	"System is going down so clean up"

	uniqueInstance closeDB
! !

!QuickNotes methodsFor: 'private' stamp: 'ArkadiuszBaranowski 8/13/2013 16:15'!
acceptNote
"DEPRECATED!!"
	"Accepts the current note if it has unaccepted edits. Thus it will be saved into DB as well."

	"contentEditor ifNotNil: [
		contentEditor hasUnacceptedEdits ifTrue: [ contentEditor textMorph acceptContents ] ]"
! !

!QuickNotes methodsFor: 'private' stamp: 'ArkadiuszBaranowski 8/13/2013 16:56'!
addNewNote: aKeyString
	"Adds a new empty note to the DB and selects it as the current one."

	currentNote := QNNote new
		keyString: aKeyString;
		content: initialContent .
	notesDB put: currentNote.
	keyStringsListed := {aKeyString}.
	selectedKeyStringIndex := 1.
	contentEditor takeKeyboardFocus.
	contentEditor setSelection: (1 to: 3)! !

!QuickNotes methodsFor: 'open/close' stamp: 'ArkadiuszBaranowski 8/13/2013 16:08'!
buildGUI
	"Builds the GUI and opens main window."
	
	|builder content search list buttonBar |
	
	builder := UITheme builder.
	search := builder
			newTextEntryFor: self
			getText: #searchText
			setText: #searchText:
			help: 'search key containing given string'.
	list := builder
			newListFor: self
			list: #keyStringsListed
			selected: #keyStringSelectedIndex
			changeSelected: #keyStringSelectedIndex:
			help: #keyStringsListed.
	buttonBar := builder newRow: {
			builder newButtonFor: self
				action: #newEntry
				label: 'New Entry'
				help: 'Add a new note.'.
			builder newButtonFor: self
				action: #rename
				label: 'Rename'
				help: 'Rename an entry.'.
			builder newButtonFor: self
				action: #deleteEntry
				label: 'Delete Entry'
				help: 'Deletes selected note.'}.
	contentEditor := builder
			newTextEditorFor: self
			getText: #content
			setText: #content:.
	contentEditor acceptOnFocusChange: true.
	content := builder newColumn: { search. list. buttonBar. contentEditor }.
	
	"(content openInWindowLabeled: 'Quick Notes') extent: 400@500."
	mainWindow := builder newWindowFor: self title: 'Quick Notes'.
	mainWindow addMorph: content fullFrame: (LayoutFrame fractions: (0@0 corner: 1@1)).
	"mainWindow extent: 400@500."
! !

!QuickNotes methodsFor: 'private' stamp: 'ArkadiuszBaranowski 8/12/2013 22:20'!
changed

	self changed: #searchText;
		changed: #keyStringsListed;
		changed: #hasSelectedKeyIndex;
		changed: #keyStringSelectedIndex;
		changed: #content
! !

!QuickNotes methodsFor: 'open/close' stamp: 'ArkadiuszBaranowski 6/13/2014 23:21'!
closeDB
	"closes the DB. Needed on finalize and SmalltalkImage shutdown."

	notesDB close.
	Transcript show: 'DB closed.'; cr.! !

!QuickNotes methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 8/8/2013 11:26'!
content
	"Returns the content of the currently selected key string."

	"Transcript show: 'content.'; cr."
	^ currentNote content ! !

!QuickNotes methodsFor: 'callback' stamp: 'ArkadiuszBaranowski 8/13/2013 19:40'!
content: aContent
	"Callback method: Sets the content of the currently selected key string and stores it in DB."

	| keyStr |
	(aContent = initialContent) not
	ifTrue: [ 
		(currentNote keyString size = 0)
		ifTrue: [ 
			keyStr := UIManager default request: 'Enter a name for the new note please.'.
			currentNote keyString: keyStr ].
		currentNote
			"keyString: (keyStringsListed at: selectedKeyStringIndex);"
			content: aContent.
		notesDB put: currentNote.
		self changed ]
	"Transcript show: 'content: ', currentNote keyString, '->',currentNote content; cr."
	! !

!QuickNotes methodsFor: 'callback' stamp: 'ArkadiuszBaranowski 8/13/2013 18:12'!
deleteEntry
	"Deletes selected note."

	| deleteKeyStr |
	deleteKeyStr := currentNote keyString.
	keyStringsListed remove: deleteKeyStr ifAbsent: [ ^ nil ].
	searchText := lastSearchText.
	selectedKeyStringIndex := 0.
	currentNote keyString: lastSearchText.
	currentNote content: initialContent.
	notesDB remove: deleteKeyStr.
	self changed.
	Transcript show: 'Note with key string ''', deleteKeyStr, ''' deleted.'; cr.! !

!QuickNotes methodsFor: 'private' stamp: 'ArkadiuszBaranowski 8/15/2013 14:34'!
dialogNoteName: initialName
	"Displays a modal dialog which asks for a note name."

	| name |
	
	name := ''.
	[ name isNil not and: [name trim = ''] ]
	whileTrue: [ name := UIManager default
		request: 'Enter a new name for selected note please.'
		initialAnswer: initialName ].
	^ name
! !

!QuickNotes methodsFor: 'open/close' stamp: 'ArkadiuszBaranowski 6/13/2014 23:21'!
finalize
	
	self closeDB.
	self class delete.
	super finalize! !

!QuickNotes methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 8/7/2013 13:13'!
hasSelectedKeyIndex

	Transcript show: 'hasSelectedKeyIndex. ', (selectedKeyStringIndex > 0) asString; cr.

	  ^ selectedKeyStringIndex > 0! !

!QuickNotes methodsFor: 'open/close' stamp: 'ArkadiuszBaranowski 6/13/2014 12:30'!
initialize 

	super initialize.
	SmalltalkImage current addToShutDownList: self class.
	searchText := 'search text'.
	lastSearchText := ''.
	initialContent := '<new note>'.
	notesDB := QNDBSQLite new.
	(self listKeyStringsMatching: '')
		ifNil: [ currentNote := QNNote new keyString: searchText;
			content: initialContent ]
		ifNotNil: [
			selectedKeyStringIndex := 1.
			self updateCurrentNote ].
	self buildGUI.

! !

!QuickNotes methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 8/8/2013 21:52'!
keyStringSelectedIndex

	^ selectedKeyStringIndex ifNil: [selectedKeyStringIndex := 0].
	"Transcript show: 'keyStringSelectedIndex ^ ', index asString; cr."
! !

!QuickNotes methodsFor: 'callback' stamp: 'ArkadiuszBaranowski 8/15/2013 14:16'!
keyStringSelectedIndex: aSmallInteger 
	"Callback method."
	
	"Transcript show: 'content selection: ', contentEditor textMorph selection asString; cr."

	selectedKeyStringIndex := aSmallInteger.
	self updateCurrentNote.
	self changed.
! !

!QuickNotes methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 8/8/2013 14:36'!
keyStringsListed

	^ keyStringsListed 
! !

!QuickNotes methodsFor: 'private' stamp: 'ArkadiuszBaranowski 8/9/2013 22:06'!
listKeyStringsMatching:  searchString
	"Gets all key strings matching searchText from database and stores them
	in instance variable keyStringsListed. Will return nil and leave keyStringsListed untouched if
	no matching key strings are found."

	| matchingNotes |

	matchingNotes := notesDB get: searchString .
	^ matchingNotes ifNotNil: [
		keyStringsListed := OrderedCollection newFrom:
			(matchingNotes collect: [ :each | each keyString  ] ) ].

	"Transcript show: 'keyStrings found:'; cr.
	Transcript show: keyStringsListed asString; cr."
! !

!QuickNotes methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 8/10/2013 13:06'!
mainWindow

	^ mainWindow! !

!QuickNotes methodsFor: 'callback' stamp: 'ArkadiuszBaranowski 8/15/2013 14:31'!
newEntry
	"Creates a new note."

	| keyStr |
	
	keyStr := self dialogNoteName: searchText.

	keyStr ifNotNil: [ 
		lastSearchText := searchText.
		searchText := keyStr.
		self addNewNote: keyStr.
		self changed ]
	
	! !

!QuickNotes methodsFor: 'callback' stamp: 'ArkadiuszBaranowski 8/15/2013 14:29'!
rename
	"comment stating purpose of message"

	| oldKeyStr newKeyStr |

	(selectedKeyStringIndex isNil not and: [ selectedKeyStringIndex > 0 ])
	ifTrue: [ 

		oldKeyStr := keyStringsListed at: selectedKeyStringIndex.
		newKeyStr := self dialogNoteName: oldKeyStr.
		newKeyStr ifNotNil: [ 
			keyStringsListed at: selectedKeyStringIndex put: newKeyStr.
			notesDB rename: oldKeyStr as: newKeyStr.
			self changed ] ]
! !

!QuickNotes methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 8/7/2013 12:29'!
searchText
	"returns the string from the search text field."

	"Transcript show: 'searchText.'; cr."
	^ searchText! !

!QuickNotes methodsFor: 'callback' stamp: 'ArkadiuszBaranowski 8/15/2013 13:44'!
searchText: aString
	"Callback method: Sets the search text instance variable to aString and
	performs search. Creates a new note if search yields no result."

	searchText := aString.
	selectedKeyStringIndex := 0.
	"Transcript	show: 'searchText: ', aString; cr."

	"Update dependent instance variables."
	(self listKeyStringsMatching: aString)
		ifNotNil: [ keyStringsListed size = 1
			ifTrue: [ selectedKeyStringIndex := 1 ]
			ifFalse: [ lastSearchText := aString ]. "Copy for later; see method deleteEntry."
			self updateCurrentNote ]
		ifNil: [ self addNewNote: aString ].

	self changed! !

!QuickNotes methodsFor: 'private' stamp: 'ArkadiuszBaranowski 8/13/2013 10:15'!
updateCurrentNote
	"Gets the note from DB corresponding to the currently selected key string in list."

	self acceptNote.

	(selectedKeyStringIndex isNil or: [selectedKeyStringIndex < 1])
		ifTrue: [ currentNote keyString: searchText.
			currentNote content: initialContent  ]
		ifFalse: [ searchText := keyStringsListed at: selectedKeyStringIndex.
			currentNote := (notesDB get: searchText) first. ]
! !

!QuickNotes methodsFor: 'open/close' stamp: 'ArkadiuszBaranowski 8/14/2013 14:14'!
windowIsClosing 
	
	self finalize.
	super windowIsClosing ! !

Object subclass: #QNDBSQLite
	instanceVariableNames: 'connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuickNotes'!

!QNDBSQLite commentStamp: '<historical>' prior: 0!
A QNDBSQLite object is a DB interface to a SQLite DB.

Instance Variables
	connection:		<Object>

notesDB
	- xxxxx
!

!QNDBSQLite methodsFor: 'open' stamp: 'ArkadiuszBaranowski 8/13/2013 10:04'!
checkConnection
	"Connection may be broken due to a system restart. Therefore check it and
	reinitialize it if necessary."

	| result |
	result := connection executeQuery: 'select count(*) as SIZE from NOTES'.
	result ifEmpty: [
		Transcript show: 'DB is broken!!.'; cr.
		Exception signal: 'DB connection is broken!! Restart application please.' ].! !

!QNDBSQLite methodsFor: 'open' stamp: 'ArkadiuszBaranowski 8/8/2013 23:44'!
close

	connection close.! !

!QNDBSQLite methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 8/12/2013 23:38'!
get: aString
	"Returns a collection of notes with aString in key string. If no note exists with such
	a string nil will be returned."

	| sqlQuery result |

	self checkConnection.
	sqlQuery := 'select KEY_STRING, NOTE from NOTES '.
	aString = ''
		ifFalse: [ 
			sqlQuery := sqlQuery,
				'where KEY_STRING like ''%', aString, '%''' ].
	result := connection executeQuery: sqlQuery.
	result size = 0 ifTrue: [ ^ nil ].
	^ result collect: [ :row | QNNote new
		keyString: (row at: #KEY_STRING);
		content: (row at: #NOTE) ].
! !

!QNDBSQLite methodsFor: 'open' stamp: 'ArkadiuszBaranowski 9/1/2013 14:12'!
initialize

	| dbPathString |
	super initialize.
	connection := SQLiteConnection new.
	dbPathString := DiskStore currentFileSystem workingDirectory pathString,
		DiskStore delimiter asString, 'QuickNotes.db'.
	connection fileName: dbPathString.
	connection open.
	connection executeQuery: 

'create table if not exists NOTES (
	ID integer primary key autoincrement,
	KEY_STRING text,
	NOTE text,
	EDIT_DATE date default current_timestamp) '! !

!QNDBSQLite methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 8/10/2013 02:04'!
keyStrings
	"comment stating purpose of message"

	^ OrderedCollection newFrom: (
		(self get: '') collect: [ :aNote | aNote keyString ] ).! !

!QNDBSQLite methodsFor: 'updating' stamp: 'ArkadiuszBaranowski 8/13/2013 19:31'!
put: aNote 
	"Adds a new note or changes an existing one."
	
	| sqlQuery matchingNotes content |
	
	self checkConnection .
	sqlQuery := 
		'select KEY_STRING, NOTE from NOTES ',
		'where KEY_STRING = ''', aNote keyString, ''''.
	matchingNotes := connection executeQuery: sqlQuery.

	matchingNotes size = 0
		ifTrue: [ "insert new row"
			sqlQuery := 'insert into NOTES (KEY_STRING, NOTE) ',
				'values (''', aNote keyString , ''', '''')'. ]

		ifFalse: [ "update existing row"
			content := aNote content copyReplaceAll: '''' with: ''''''. "We need to escape single quotes"
			sqlQuery := 'update NOTES set NOTE = ''', content,
				''', EDIT_DATE = current_timestamp where KEY_STRING = ''', aNote keyString, '''' ].

		connection executeQuery: sqlQuery! !

!QNDBSQLite methodsFor: 'updating' stamp: 'ArkadiuszBaranowski 8/12/2013 18:50'!
remove: aKeyString
	"Removes the note with the specified key string."

	| sqlQuery |
	
	self checkConnection .

	sqlQuery := 'delete from NOTES ',
		'where KEY_STRING = ''', aKeyString, ''''.
	connection executeQuery: sqlQuery.
! !

!QNDBSQLite methodsFor: 'updating' stamp: 'ArkadiuszBaranowski 9/2/2013 16:39'!
rename: oldKeyStr as: newKeyStr
	"Renames a note named oldKeyStr as newKeyStr."

	| sqlQuery |

	self checkConnection .
	sqlQuery := 'update NOTES set KEY_STRING = <1p> where KEY_STRING = <2p>'
		expandMacrosWith: newKeyStr with: oldKeyStr.
	connection executeQuery: sqlQuery.
	Transcript show: 'rename query: ', sqlQuery; cr
! !

!QNDBSQLite methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 8/12/2013 18:52'!
size
	"comment stating purpose of message"

	| sqlQuery result |

	self checkConnection .
	sqlQuery := 'select count(*) as size from NOTES'.
	result := connection executeQuery: sqlQuery.
	^ result first at: #size! !

!QNDBSQLite methodsFor: 'debugging' stamp: 'ArkadiuszBaranowski 9/1/2013 14:11'!
transcriptContent

	| notes |
	Transcript show: '----------------- Table NOTES -------------------------'; cr.
	Transcript show: 'ID<t>KEY_STRING<t>NOTE<t>EDIT_DATE' expandMacros; cr.
	notes := connection executeQuery: 'select * from NOTES'.
	notes do:
		[ :row | | str |
		str := '<1s><t><2s><t><3s><t><t><4s><n>' expandMacrosWith:
			(row at: #ID) with: (row at: #KEY_STRING) with: (row at: #NOTE) with: (row at: #EDIT_DATE).
		Transcript show: str ].
	Transcript show: '----------------------------------------------------------------'; cr.
! !

Object subclass: #QNDBSimple
	instanceVariableNames: 'notesCollection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuickNotes'!

!QNDBSimple commentStamp: '<historical>' prior: 0!
A QNDBSimple object is a simple reference implementation of a Quick Notes database
abstract layer.
If you want to implement your own Quick Notes database abstract layer then you will
have to provide all the methods of this class with the same functionality.!

!QNDBSimple methodsFor: 'open' stamp: 'ArkadiuszBaranowski 9/2/2013 16:29'!
close
	"Close the database."
	notesCollection := nil! !

!QNDBSimple methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 8/8/2013 23:48'!
get: aString
	"Returns a collection of notes with aString in keyString. If no note exists with such
	a string nil will be returned."

	| result |
	aString = ''
		ifTrue: [
			result := notesCollection ]
		ifFalse: [ 
			result := notesCollection select: [ :aNote | (aNote keyString findString: aString) > 0 ] ].
	result size = 0 ifTrue: [ ^ nil ].
	^ result! !

!QNDBSimple methodsFor: 'open' stamp: 'ArkadiuszBaranowski 8/8/2013 23:48'!
initialize

	super initialize.
	notesCollection := OrderedCollection new.! !

!QNDBSimple methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 8/8/2013 23:48'!
keyStrings
	"Returns a collection with all key strings in DB."

	^ notesCollection collect: [ :aNote | aNote keyString  ].! !

!QNDBSimple methodsFor: 'updating' stamp: 'ArkadiuszBaranowski 8/8/2013 23:48'!
put: aNote 
	"Adds a new note or changes an existing one."
	
	| matchingNotes |
	
	matchingNotes := notesCollection select: [ :each | each keyString = aNote keyString ].
	matchingNotes size = 0
		ifTrue: [ notesCollection add: aNote ]
		ifFalse: [ matchingNotes first content: aNote content ]! !

!QNDBSimple methodsFor: 'updating' stamp: 'ArkadiuszBaranowski 8/8/2013 23:49'!
remove: aKeyString
	"Removes the note with the specified key string."

	notesCollection copy do: [ :each |
		each keyString = aKeyString ifTrue: [ notesCollection remove: each] ]! !

!QNDBSimple methodsFor: 'updating' stamp: 'ArkadiuszBaranowski 9/2/2013 16:39'!
rename: oldKeyString as: newKeyString
	"Renames a note named oldKeyStr as newKeyStr."

	Transcript show: 'Not implemented yet.'; cr! !

!QNDBSimple methodsFor: 'accessing' stamp: 'ArkadiuszBaranowski 9/2/2013 16:40'!
size
	"Returns the number of notes in DB."

	^ notesCollection size! !

Object subclass: #QNNote
	instanceVariableNames: 'keyString content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuickNotes'!

!QNNote commentStamp: '<historical>' prior: 0!
A Note is a text note identified by a unique string.

Instance Variables
	content:		the text content of the note
	keyString:		a unique string identifying the note

content
	- xxxxx

keyString
	- xxxxx
!

!QNNote methodsFor: 'as yet unclassified' stamp: 'ArkadiuszBaranowski 8/2/2013 15:03'!
content

	^content! !

!QNNote methodsFor: 'as yet unclassified' stamp: 'ArkadiuszBaranowski 8/2/2013 15:04'!
content: text

	content := text! !

!QNNote methodsFor: 'as yet unclassified' stamp: 'ArkadiuszBaranowski 8/2/2013 15:05'!
keyString

	^keyString! !

!QNNote methodsFor: 'as yet unclassified' stamp: 'ArkadiuszBaranowski 8/2/2013 15:05'!
keyString: text

	keyString := text! !
